import pgzrun
from pgzero.builtins import Actor, keyboard, mouse
from pgzero.clock import clock
import random
import math
import time

WIDTH, HEIGHT = 1280, 800
MAX_SPEED = 0.5
MIN_SIZE = 0

# 显式初始化 Pygame
# 尝试加载背景图片

class Scoreboard:
    def __init__(self):
        self.score = 0

    def draw(self):
        screen.draw.text(f"Score: {self.score}", center=(WIDTH // 2, HEIGHT - 30), fontsize=30, color="white")

    def update_score(self, points):
        self.score += points

class StartScreen:
    def __init__(self):
        self.active = True

    def draw(self):
        screen.clear()
        screen.blit("background.jpg", (0, 0))  # 假设有一个开始界面的背景图片
    # 游戏名称
        screen.draw.text("KILL VIRUS", center=(WIDTH // 2, HEIGHT // 2 - 50), color="white", fontsize=180, owidth=1.5, ocolor="black")
    # "Click to start" 文本
        screen.draw.text("Click to start", center=(WIDTH // 2, HEIGHT // 2 + 250), color="white", fontsize=50, owidth=1.0, ocolor="black")

    def handle_input(self, pos, button):
        if mouse.LEFT == button:
            self.active = False

class EndScreen:
    def __init__(self):
        self.active = False

    def draw(self, score):
        screen.clear()
        screen.blit("end_background.jpg", (0, 0))  # 假设有一个结束界面的背景图片
        screen.draw.text(f"Final Score: {score}", center=(WIDTH // 2, HEIGHT // 2 - 100), color="white", fontsize=100, owidth=1.5, ocolor="black")
        screen.draw.text("Press SPACE to Play Again", center=(WIDTH // 2, HEIGHT // 2), color="white", fontsize=50, owidth=1.0, ocolor="black")
        screen.draw.text("Press ESC to Exit", center=(WIDTH // 2, HEIGHT // 2 + 200), color="white", fontsize=50, owidth=1.0, ocolor="black")
        # 添加游戏制作人信息
        screen.draw.text("Game by Jerry Cai, Genesis Yang\nGuided by Prof. Guarnera", center=(WIDTH // 2, HEIGHT // 2 + 250), color="white", fontsize=50, owidth=1.0, ocolor="black")

    def handle_input(self, pos, button):
        if mouse.LEFT == button:
            return True  # 返回 True 表示重新开始游戏
        return False

class Virus:
    def __init__(self, stage):
        self.actor = Actor('virus_' + str(stage))
        self.size = self.actor.width
        self.stage = stage
        self.actor.pos = [random.uniform(self.size, WIDTH - self.size), random.uniform(self.size, HEIGHT - self.size)]
        
        # 根据病毒大小设置不同的速度
        if stage == 0:  # 小病毒
            speed_factor = 2.5
        elif stage == 1:  # 中病毒
            speed_factor = 1.5
        else:  # 大病毒
            speed_factor = 1

        self.speed = [random.uniform(-MAX_SPEED * speed_factor, MAX_SPEED * speed_factor), random.uniform(-MAX_SPEED * speed_factor, MAX_SPEED * speed_factor)]
    def update(self):
        # 更新病毒的位置
        self.actor.x += self.speed[0]
        self.actor.y += self.speed[1]

        # 检查并处理边界碰撞
        if self.speed[0] < 0 and self.actor.x < self.size:
            self.speed[0] *= -1
        if self.speed[0] > 0 and self.actor.x > WIDTH - self.size:
            self.speed[0] *= -1

        if self.speed[1] < 0 and self.actor.y < self.size:
            self.speed[1] *= -1
        if self.speed[1] > 0 and self.actor.y > HEIGHT - self.size:
            self.speed[1] *= -1

class Game:

    def __init__(self):
        self.hit = False
        self.start_screen = StartScreen()
        self.end_screen = EndScreen()  # 初始化结束屏幕
        self.scoreboard = Scoreboard()
        self.victory_music_played = False
        self.timer = 60  # 游戏时间为 60 秒
        self.speed_boosted = False
        self.start_time = time.time()
        self.game_over = False
        self.viruses = []
        for _ in range(random.randint(2, 6)):  # 大病毒
            self.viruses.append(Virus(2))
        for _ in range(random.randint(4, 8)):  # 中病毒
            self.viruses.append(Virus(1))
        for _ in range(random.randint(4, 9)):  # 小病毒
            self.viruses.append(Virus(0))
        self.background = 'background.jpg'
        music.play("background_music")  # 播放背景音乐，它将自动循环播放   
        
    def reset_game(self):
        self.hit = False
        self.start_screen = StartScreen()
        self.end_screen = EndScreen()
        self.victory_music_played = False
        self.scoreboard = Scoreboard()
        self.timer = 60
        self.speed_boosted = False
        self.start_time = time.time()
        self.game_over = False
        self.viruses = []
        for _ in range(random.randint(2, 6)):  # 大病毒
            self.viruses.append(Virus(2))
        for _ in range(random.randint(4, 8)):  # 中病毒
            self.viruses.append(Virus(1))
        for _ in range(random.randint(4, 9)):  # 小病毒
            self.viruses.append(Virus(0))
        self.background = 'background.jpg'  # 保持背景图片设置
        music.play("background_music")  # 重新播放背景音乐
   
    def game_update(self):
        if not self.start_screen.active and not self.game_over:
            self.timer -= 1 / 60  # 每帧减少的时间
            if self.timer <= 0:
                self.game_over = True  # 设置游戏结束标志
                if not self.victory_music_played:
                    music.stop()  # 停止背景音乐
                    sounds.victory_music.play()  # 播放胜利音乐
                    clock.schedule_unique(self.stop_victory_music, 5.0)  # 假设音乐长度为5秒
                    self.victory_music_played = True

            if self.timer <= 30 and not self.speed_boosted:
                self.boost_virus_speed()
                self.speed_boosted = True

            self.move_viruses()
            self.check_collisions()

    def stop_victory_music(self):
        sounds.victory_music.stop()
                
    def game_draw(self):
        screen.blit(self.background, (0, 0))  # 绘制背景
        if self.start_screen.active:
            self.start_screen.draw()
        elif not self.game_over:  # 只有游戏未结束时才绘制病毒和计分板
            for virus in self.viruses:
                virus.actor.draw()
            self.scoreboard.draw()
        if self.game_over:
            self.end_screen.draw(self.scoreboard.score)
            
    def on_mouse_down(self, pos, button):
        if self.start_screen.active:
            self.start_screen.handle_input(pos, button)
        elif not self.game_over:
        # 游戏进行中时处理鼠标点击
            self.handle_input(pos, button)

    def on_key_down(self, key):
        if key == keys.ESCAPE:
            if self.game_over:
                exit()  # 退出游戏
        elif key == keys.SPACE:
            if self.game_over:
                self.reset_game()  # 按空格键重新开始游戏

    def handle_input(self, pos, button):
        if not self.game_over:  # 只有游戏进行中时才响应
            if mouse.LEFT == button:
                for virus in self.viruses:
                    if virus.actor.collidepoint(pos):
                        new_stage = virus.stage - 1
                        # 根据病毒大小更新分数
                        if virus.stage == 0:  # 小病毒
                            self.scoreboard.update_score(3)
                        elif virus.stage == 1:  # 中病毒
                            self.scoreboard.update_score(5)
                        elif virus.stage == 2:  # 大病毒
                            self.scoreboard.update_score(10)

                        # 分裂成两个更小的病毒
                        if new_stage > -1:
                            self.viruses.extend([self.create_virus(new_stage), self.create_virus(new_stage)])
                        self.viruses.remove(virus)
                        break
    
    def boost_virus_speed(self):
        for virus in self.viruses:
            virus.speed[0] *= 1.5
            virus.speed[1] *= 1.5
                    
    def move_viruses(self):
        #global score
        for virus in self.viruses:
            virus.update()

    def check_collisions(self):#fuse
        #global score
        did_collide = True
        
        new_viruses = []
        
        while did_collide:
            did_collide = False
            
            virus_count = len(self.viruses)
            for index1 in range(0, virus_count):
                v1 = self.viruses[index1]
                v1_center = v1.actor.center
                
                for index2 in range(index1 + 1, virus_count):
                    v2 = self.viruses[index2]
                    v2_center = v2.actor.center
                    
                    # Fast distance check (distance squared to avoid the sqrt)
                    if (v1_center[0] - v2_center[0])**2 + (v1_center[1] - v2_center[1])**2 < 35**2:
                        if v1.stage == v2.stage and v1.stage != 2:
                            did_collide = True
                            self.viruses.remove(v1)
                            self.viruses.remove(v2)
                            new_viruses.append(self.create_virus(v1.stage+1))
                            break
                if did_collide:
                    break
        
        self.viruses.extend(new_viruses)        
             
    def create_virus(self, size):
        return Virus(size)
    
    def handle_input(self, pos, button):
        if mouse.LEFT == button:
            for virus in self.viruses:
                if virus.actor.collidepoint(pos):
                    new_stage = virus.stage - 1
                    self.scoreboard.update_score(10 if virus.stage == 2 else 5 if virus.stage == 1 else 3)
                    sounds.virus_hit.play()  # 播放击中病毒的音效
                    if new_stage > -1:
                        self.viruses.extend([self.create_virus(new_stage), self.create_virus(new_stage)])
                    self.viruses.remove(virus)
                    break

    def spawn_virus(self):
        # 随机选择病毒类型
        rand = random.randint(1, 100)
        if rand <= 20:  # 20% 概率生成大病毒
            virus = self.create_virus(2)
        elif rand <= 70:  # 50% 概率生成中病毒
            virus = self.create_virus(1)
        else:  # 30% 概率生成小病毒
            virus = self.create_virus(0)
        self.viruses.append(virus)
    
#             screen.draw.circle((virus.actor.x, virus.actor.y), virus.size - 1, virus.actor.color)
#             for i in range(N_PIECES):
#                 angle = math.pi * 2 / N_PIECES * i
#                 x = virus.actor.x + virus.size * math.cos(angle)
#                 y = virus.actor.y + virus.size * math.sin(angle)
#                 screen.draw.circle((x, y), 1, virus.actor.color)

def draw_text(text, x, y, color):
    screen.draw.text(text, (x, y), color=color, fontsize=30)

virus_game = Game()

#virus_game = Game()
def update():
    # 调用 Game 实例的更新方法
    virus_game.game_update()

def draw():
    # 调用 Game 实例的绘制方法
    virus_game.game_draw()

def on_key_down(key):
    virus_game.on_key_down(key)

def on_mouse_down(pos, button):
    virus_game.on_mouse_down(pos, button)

clock.schedule_interval(virus_game.spawn_virus, 2)

pgzrun.go()