import pyxel
import random
import math
import time

W, H = 150, 150
N_PIECES = 12
MAX_SPEED = 0.5
MIN_SIZE = 1 #终极难度

class Virus:
    def __init__(self):
        self.size = random.uniform(3,10) #随机尺寸
        self.pos = [random.uniform(self.size, W - self.size), random.uniform(self.size, H - self.size)]
        self.speed = [random.uniform(-MAX_SPEED, MAX_SPEED), random.uniform(-MAX_SPEED, MAX_SPEED)]
        self.color = random.randint(1,15)
        
    def update(self):
        self.pos[0] += self.speed[0]
        self.pos[1] += self.speed[1]
        #x direction
        if self.speed[0] < 0 and self.pos[0] < self.size :#moving toward left
            self.speed[0]  *= -1
        if self.speed[0] >0 and self.pos[0] > W - self.size:#moving toward right
            self.speed[0] *= -1
        #y direction
        if self.speed[1] < 0 and self.pos[1] < self.size :#moving upward
            self.speed[1]  *= -1
        if self.speed[1] >0 and self.pos[1] > H - self.size:#moving downward
            self.speed[1] *= -1   
    
class Game:
    
    def __init__(self):
        pyxel.init(W, H, name='Kill Virus Pixel Game')
        pyxel.mouse(True)
        self.hit = False #未被点击
        self.start_time = time.time() #记时
        self.game_over = False
        self.viruses = [Virus() for _ in range(10)] 
        pyxel.run(self.update, self.draw)
        
        
    def update(self):
        
        virus_count = len(self.viruses)
        # Split 
        if pyxel.btnp(pyxel.MOUSE_LEFT_BUTTON):#计算鼠标和病毒中心点的关系
            self.hit = True #被点击
            for i in range(virus_count):
                virus = self.viruses[i]
                dx = virus.pos[0] - pyxel.mouse_x #x轴差
                dy = virus.pos[1] - pyxel.mouse_y #y轴差
                if dx ** 2 + dy**2 < virus.size ** 2: #在圆内
                    new_size = (virus.size ** 2 / N_PIECES) ** 0.5 #分裂后的大小
                    #如果大于最小可杀尺寸，则会分裂
                    if new_size > MIN_SIZE:#分裂出新的
                        for j in range(N_PIECES):
                            angle = math.pi * 2 * j / N_PIECES
                            new_virus = Virus()
                            new_virus.size = new_size
                            new_virus.pos[0] = virus.pos[0] + (virus.size + new_size) * math.cos(angle)
                            new_virus.pos[1] = virus.pos[0] + (virus.size + new_size) * math.sin(angle)
                            new_virus.speed[0] = MAX_SPEED * math.cos(angle)
                            new_virus.speed[1] = MAX_SPEED * math.sin(angle)
                            self.viruses.append(new_virus)
                    del self.viruses[i]
                    break
                
        #Fuse together
        virus_count = len(self.viruses)
        for i in range (virus_count - 1, -1, -1):#倒计数，每次融合都会少一个
            vi = self.viruses[i]#第i个
            vi.update()
            for j in range(i -1, -1, -1):
                vj = self.viruses[j]#第j个，x和y方向的差
                dx = vi.pos[0] - vj.pos[0]
                dy = vi.pos[1] - vj.pos[1]
                total_size = vi.size + vj.size
                
                if dx**2 + dy**2 < total_size ** 2:#距离小于半径之和，就融合
                    new_virus = Virus()
                    new_virus.size = (vi.size ** 2 + vj.size ** 2) ** 0.5 #新圆半径
                    new_virus.pos[0] = vi.pos[0] * vi.size/total_size + vj.pos[0] * vj.size/total_size #哪个大整体位置偏向哪个中心
                    new_virus.pos[1] = vi.pos[1] * vi.size/total_size + vj.pos[1] * vj.size/total_size 
                    new_virus.speed[0] = vi.speed[0] * vi.size/total_size + vj.speed[0] * vj.size/total_size
                    new_virus.speed[1] = vi.speed[1] * vi.size/total_size + vj.speed[1] * vj.size/total_size 
                    self.viruses.append(new_virus)#加入新病毒
                    del self.viruses[i]#删去旧的俩
                    del self.viruses[j]
                    break
                
        for virus in self.viruses:
            virus.update()
    
    def draw(self):
        
        pyxel.cls(0)#清空屏幕
        if not self.hit:#点击后字会消失
            pyxel.text(40, 70, 'KILL VIRUS!', pyxel.frame_count % 16)#变化16种颜色
            
        if not self.game_over and len(self.viruses) == 0:#病毒被清干净，游戏结束
            self.game_over = True
            self.total_time = time.time() - self.start_time
        
        if self.game_over:
            pyxel.text(50, 60, 'YOU WIN!', pyxel.frame_count % 16)
            pyxel.text(50, 70, 'TIME: {}s'.format(int(self.total_time)), pyxel.frame_count % 16)
        
        for virus in self.viruses:
            pyxel.circ(virus.pos[0], virus.pos[1], virus.size - 1, virus.color)
            for i in range (N_PIECES):
                angle = math.pi * 2 / N_PIECES * i
                pyxel.circ(virus.pos[0] + virus.size * math.cos(angle), virus.pos[1] + virus.size * math.sin(angle), 1, virus.color)
                
            
        
game = Game()